@@ -1,7 +1,6 @@
 use std::path::{Path, PathBuf};
 use std::time::Duration;
 
-#[cfg(feature = "completions")]
 use anyhow::anyhow;
 use clap::{
     error::ErrorKind, value_parser, Arg, ArgAction, ArgGroup, ArgMatches, Command, Parser,
@@ -18,69 +17,6 @@ use crate::filesystem;
 use crate::filter::OwnerFilter;
 use crate::filter::SizeFilter;
 
-// Type for options that don't have any values, but are used to negate
-// earlier options
-struct Negations;
-
-impl clap::FromArgMatches for Negations {
-    fn from_arg_matches(_: &ArgMatches) -> clap::error::Result<Self> {
-        Ok(Negations)
-    }
-
-    fn update_from_arg_matches(&mut self, _: &ArgMatches) -> clap::error::Result<()> {
-        Ok(())
-    }
-}
-
-impl clap::Args for Negations {
-    fn augment_args(cmd: Command) -> Command {
-        Self::augment_args_for_update(cmd)
-    }
-
-    fn augment_args_for_update(cmd: Command) -> Command {
-        cmd.arg(
-            Arg::new("no_hidden")
-                .action(ArgAction::Count)
-                .long("no-hidden")
-                .overrides_with("hidden")
-                .hide(true)
-                .long_help("Overrides --hidden."),
-        )
-        .arg(
-            Arg::new("ignore")
-                .action(ArgAction::Count)
-                .long("ignore")
-                .overrides_with("no_ignore")
-                .hide(true)
-                .long_help("Overrides --no-ignore."),
-        )
-        .arg(
-            Arg::new("ignore_vcs")
-                .action(ArgAction::Count)
-                .long("ignore-vcs")
-                .overrides_with("no_ignore_vcs")
-                .hide(true)
-                .long_help("Overrides --no-ignore-vcs."),
-        )
-        .arg(
-            Arg::new("relative_path")
-                .action(ArgAction::Count)
-                .long("relative-path")
-                .overrides_with("absolute_path")
-                .hide(true)
-                .long_help("Overrides --absolute-path."),
-        )
-        .arg(
-            Arg::new("no_follow")
-                .action(ArgAction::Count)
-                .long("no-follow")
-                .overrides_with("follow")
-                .hide(true)
-                .long_help("Overrides --follow."),
-        )
-    }
-}
-
 #[derive(Parser)]
 #[command(
     name = "fd",
@@ -104,6 +40,10 @@ pub struct Opts {
     )]
     pub hidden: bool,
 
+    /// Overrides --hidden
+    #[arg(long, overrides_with = "hidden", hide = true, action = ArgAction::SetTrue)]
+    no_hidden: (),
+
     /// Do not respect .(git|fd)ignore files
     #[arg(
         long,
@@ -114,6 +54,10 @@ pub struct Opts {
     )]
     pub no_ignore: bool,
 
+    /// Overrides --no-ignore
+    #[arg(long, overrides_with = "no_ignore", hide = true, action = ArgAction::SetTrue)]
+    ignore: (),
+
     /// Do not respect .gitignore files
     #[arg(
         long,
@@ -123,6 +67,10 @@ pub struct Opts {
     )]
     pub no_ignore_vcs: bool,
 
+    /// Overrides --no-ignore-vcs
+    #[arg(long, overrides_with = "no_ignore_vcs", hide = true, action = ArgAction::SetTrue)]
+    ignore_vcs: (),
+
     /// Do not respect .(git|fd)ignore files in parent directories
     #[arg(
         long,
@@ -205,6 +153,10 @@ pub struct Opts {
     )]
     pub absolute_path: bool,
 
+    /// Overrides --absolute-path
+    #[arg(long, overrides_with = "absolute_path", hide = true, action = ArgAction::SetTrue)]
+    relative_path: (),
+
     /// Use a long listing format with file metadata
     #[arg(
         long,
@@ -228,6 +180,10 @@ pub struct Opts {
     )]
     pub follow: bool,
 
+    /// Overrides --follow
+    #[arg(long, overrides_with = "follow", hide = true, action = ArgAction::SetTrue)]
+    no_follow: (),
+
     /// Search full abs. path (default: filename only)
     #[arg(
         long,
@@ -592,9 +548,6 @@ pub struct Opts {
     #[cfg(feature = "completions")]
     #[arg(long, hide = true, exclusive = true)]
     gen_completions: Option<Option<Shell>>,
-
-    #[clap(flatten)]
-    _negations: Negations,
 }
 
 impl Opts {
@@ -674,32 +627,14 @@ impl Opts {
         self.gen_completions
             .map(|maybe_shell| match maybe_shell {
                 Some(sh) => Ok(sh),
-                None => guess_shell(),
+                None => {
+                    Shell::from_env().ok_or_else(|| anyhow!("Unable to get shell from environment"))
+                }
             })
             .transpose()
     }
 }
 
-#[cfg(feature = "completions")]
-fn guess_shell() -> anyhow::Result<Shell> {
-    let env_shell = std::env::var_os("SHELL").map(PathBuf::from);
-    if let Some(shell) = env_shell
-        .as_ref()
-        .and_then(|s| s.file_name())
-        .and_then(|s| s.to_str())
-    {
-        shell
-            .parse::<Shell>()
-            .map_err(|_| anyhow!("Unknown shell {}", shell))
-    } else {
-        // Assume powershell on windows
-        #[cfg(windows)]
-        return Ok(Shell::PowerShell);
-        #[cfg(not(windows))]
-        return Err(anyhow!("Unable to get shell from environment"));
-    }
-}
-
 #[derive(Copy, Clone, PartialEq, Eq, ValueEnum)]
 pub enum FileType {
     #[value(alias = "f")]
@@ -808,7 +743,7 @@ impl clap::Args for Exec {
                 .allow_hyphen_values(true)
                 .value_terminator(";")
                 .value_name("cmd")
-                .conflicts_with_all(&["exec", "list_details"])
+                .conflicts_with_all(["exec", "list_details"])
                 .help("Execute a command with all search results at once")
                 .long_help(
                     "Execute the given command once, with all search results as arguments.\n\
